/**
 * File name: ExportAbstract.cpp
 * Project: Geonkick (A percussive synthesizer)
 *
 * Copyright (C) 2020 Iurie Nistor
 *
 * This file is part of Geonkick.
 *
 * GeonKick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ExportAbstract.h"

ExportAbstract::ExportAbstract(const std::filesystem::path &path,
                               ExportFormat format)
        : exportFormat{format}
        , exportPath{path}
{
}

const std::filesystem::path& ExportAbstract::getExportPath() const
{
        return exportPath;
}

const ExportAbstract::ExportFormat& ExportAbstract::format() const
{
        return exportFormat;
}

void ExportAbstract::setError(const std::string_view &error)
{
        errorMessage = error;
}

const std::string& ExportAbstract::getError() const
{
        return errorMessage;
}


bool ExportAbstract::isError() const
{
        return !errorMessage.empty();
}
