/**
 * File name: general_envelope.h
 * Project: Geonkick (A percussive synthesizer)
 *
 * Copyright (C) 2017 Iurie Nistor
 *
 * This file is part of Geonkick.
 *
 * GeonKick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef GKICK_GENERAL_ENVELOPE_H
#define GKICK_GENERAL_ENVELOPE_H

#include "envelope.h"

class Oscillator;
class DspProxy;

class GeneralEnvelope: public Envelope
{
 public:

  GeneralEnvelope(DspProxy *dsp, const RkRect &area);
  double envelopeLength(void) const override;
  void setEnvelopeLengh(double len) override;
  void updateEnvelope() override;
  void updatePoints() override;

 protected:
  void pointAddedEvent(const EnvelopePoint &point) override;
  void pointUpdatedEvent(unsigned int index, const EnvelopePoint &point) override;
  void pointRemovedEvent(unsigned int index) override;
  double envelopeAmplitude(void) const override;

 private:
          DspProxy *dspProxy;
};

#endif
